package com.fzm.common.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.fzm.common.entity.Order;
import com.fzm.common.entity.dto.CopyrightDTO;
import com.fzm.common.entity.dto.CopyrightOrderDto;
import com.fzm.common.entity.dto.NftOrderDto;
import com.fzm.common.entity.dto.OrderDto;
import com.fzm.common.service.ApplyOrderService;
import com.fzm.common.service.CopyrightApplyService;
import com.fzm.common.service.NftService;
import com.fzm.common.service.OrderService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author wt
 * @date 2022/5/23
 */
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class ApplyOrderServiceImpl implements ApplyOrderService {
    private final CopyrightApplyService copyrightApplyService;
    private final NftService nftService;
    private final OrderService orderService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Order submitOrder(CopyrightOrderDto copyrightOrderDto) {
        CopyrightDTO copyrightDTO = BeanUtil.copyProperties(copyrightOrderDto, CopyrightDTO.class);
        Integer productId = copyrightApplyService.submit(copyrightDTO);
        OrderDto orderDto = BeanUtil.copyProperties(copyrightOrderDto, OrderDto.class);
        orderDto.setProductId(productId);
        return orderService.createOrder(orderDto);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Order updateOrder(CopyrightOrderDto copyrightOrderDto) {
        CopyrightDTO copyrightDTO = BeanUtil.copyProperties(copyrightOrderDto, CopyrightDTO.class);
        Integer productId = copyrightApplyService.update(copyrightDTO);
        OrderDto orderDto = BeanUtil.copyProperties(copyrightOrderDto, OrderDto.class);
        orderDto.setProductId(productId);
        return orderService.createOrder(orderDto);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Order nftOrder(NftOrderDto nftOrderDto) {
        Integer nftId = nftService.saveNft(nftOrderDto);
        OrderDto orderDto = BeanUtil.copyProperties(nftOrderDto, OrderDto.class);
        orderDto.setProductId(nftId);
        return orderService.createOrder(orderDto);
    }
}
